#include "complex.h"
#include "memory_mgr.h"
#include <cassert>

using namespace std;


extern BlockMemoryManager<Complex> ComplexMemoryManager;



int main() {
  // number of objects to create
  const int N=78;
  // constant used to assert result
  const double expected_sum=3003;

  // used to compute result
  double sum;

  // array of Complex allocated one at a time
  Complex **array;


  // create array
  array=new Complex * [N];

  // allocate each element
  cout << hex;
  for (int i=0;i<N;++i) {
    array[i]=new Complex((double) i, (double) i);
    cout << "address of object " << i << " : " << array[i] << endl;
  }
  cout << dec;

  // compute result
  sum=0.0;
  for (int i=0;i<N;++i) {
    sum+=array[i]->get_r();
  }
  cout << "sum = " << sum << endl;
  assert(sum==expected_sum);


  // free all objects
  for (int i=0;i<N;++i) {
    delete array[i];
  }

  // allocate a second time to see if it works
  // we now use the objects that have been freed
  for (int i=0;i<N;++i) {
    array[i]=new Complex((double) i, (double) i);
  }

  // compute result
  sum=0.0;
  for (int i=0;i<N;++i) {
    sum+=array[i]->get_r();
  }
  cout << "sum = " << sum << endl;
  assert(sum==expected_sum);

  
  cout << endl << "sizeof(Complex) = " << sizeof(Complex) << endl;

  // use array allocator
  Complex *table;
  table=new Complex[20];

  // compute result
  sum=0.0;
  for (int i=0;i<20;++i) {
    sum+=table[i].get_r();
  }
  cout << "sum = " << sum << endl;
  assert(sum==20.0);

  delete [] table;

  // return number of objects allocated
  cout << "number of objects created : " << ComplexMemoryManager.get_count_objects() << endl;

  ComplexMemoryManager.info();

  return 0;
}

